package cn.wch.meshdemo

import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import cn.wch.meshdemo.base.NodeBaseActivity
import cn.wch.meshdemo.constant.IntentConstant
import cn.wch.meshdemo.core.NodeViewModelKt
import cn.wch.meshdemo.databinding.ActivityConfigNetBinding
import cn.wch.meshdemo.storage.NodeItem
import cn.wch.meshdemo.storage.StorageUtil
import cn.wch.meshdemo.utils.ConvertUtil
import cn.wch.meshdemo.utils.LogTool
import cn.wch.meshlib.core.MeshServer
import cn.wch.meshlib.frame.ack.AckFrame
import cn.wch.meshlib.frame.ack.AckType
import com.touchmcu.ui.DialogUtil

//配网
class ConfigNetActivity: NodeBaseActivity() {

    lateinit var binding: ActivityConfigNetBinding
    var macAddress:String?=null

    override fun setView() {
        binding= ActivityConfigNetBinding.inflate(layoutInflater)
        setContentView(binding.root)
        binding.toolbar.title="配置节点"
        setSupportActionBar(binding.toolbar)
        supportActionBar?.let {
            it.setDisplayHomeAsUpEnabled(true)
            it.setHomeButtonEnabled(true)
        }
    }

    override fun initWidget() {
        intent?.let {
            macAddress=it.getStringExtra(IntentConstant.SCAN_SELECT_DEVICE)
        }
        binding.container.btnConfig.setOnClickListener {
            startConfigNode()
        }
        val configKey = StorageUtil.getConfigKey()?: ""
        binding.container.tvInputKey.setText(configKey)
        connect()
    }


    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main,menu)
        menu?.findItem(R.id.connect)?.isVisible=!isConnected
        menu?.findItem(R.id.disconnect)?.isVisible=isConnected
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val itemId = item.itemId
        when(itemId){
            R.id.connect->{
                connect()
            }
            R.id.disconnect->disconnectBLE(false)
            android.R.id.home->{
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        if(isConnected){
            DialogUtil.getInstance().showSimpleDialog(this,"需要先断开与当前蓝牙的连接，是否断开?","断开","取消",object :DialogUtil.IResult{
                override fun onContinue() {
                    disconnectBLE(false)
                }

                override fun onCancel() {

                }
            })
        }else{
            super.onBackPressed()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        LogTool.d("ConfigNetActivity destroy")
    }

    private fun connect(){
        if(macAddress==null){
            showToast("mac address is null")
            return
        }
        connectBLE(macAddress!!)
    }
    private fun startConfigNode(){
        if(!isBLEConnected()){
            showToast("尚未连接到蓝牙设备")
            return
        }
        val key = binding.container.tvInputKey.text.toString()
        val address = binding.container.tvInputAddress.text.toString()
        val name = binding.container.tvInputName.text.toString()
        //check key
        if(!key.matches(Regex("([0-9|a-f|A-F]{2}){16}"))){
            showToast("密钥必须为合法的32字节")
            return
        }
        //check address
        if(!address.matches(Regex("([0-9|a-f|A-F]{2}){2}"))){
            showToast("地址必须为合法的2字节")
            return
        }
        //check name
        if(name.isNullOrEmpty()){
            showToast("节点名称为空")
            return
        }
        //parse key and address
        val keyData = ConvertUtil.fromHexString2ByteArray(key)
        val addressData = address.toInt(16)
        if((addressData)>0x00007fff){
            showToast("节点地址不大于7FFF")
            return
        }
        if(StorageUtil.checkNodeHasExist(addressData)){
            //节点已经存在
            showToast("节点地址已经存在")
            return
        }

        val ivIndex = StorageUtil.getIvIndex()
        val flag = StorageUtil.getFlag()

        var commandLists:MutableList<(()->AckFrame?)> = mutableListOf({
            MeshServer.getInstance().setNetworkInfo(ivIndex,flag.toByte())
        },{
            MeshServer.getInstance().meshProvision(keyData,addressData)
        })


        NodeViewModelKt.model.rxjavaExecuteAny(this,"正在配置节点",commandLists,{
            var ret=true;
            if(it.ackType==AckType.PROVISION_INFO){
                ret= it.statusCode==0
            }
            return@rxjavaExecuteAny ret
        },{
            if(it.ackType==AckType.PROVISION){
                val addressString = ConvertUtil.getNodeHexString(it.netAddress)

                showToast("设置节点${addressString}${if(it.statusCode==0) "成功" else "失败"}")
                if(it.statusCode==0 && it.netAddress==addressData){
                    saveToLocalAndExit(key,addressData,name)
                }
            }else if(it.ackType==AckType.PROVISION_INFO){
                showToast("设置网络信息${if(it.statusCode==0) "成功" else "失败"}")
            } else{
                showToast("error ack")
            }
        },{
            LogTool.d(it.message)
            showToast(it.message)},{})
    }


    fun saveToLocalAndExit(key:String,netAddress:Int,name:String){
        //when config success,save message
        StorageUtil.putNode(NodeItem(name,netAddress,false))
        //save key
        StorageUtil.setConfigKey(key)
        //disconnect
        disconnectBLE(true)
        //leave
        startActivity(Intent(this,NodeActivity::class.java))
    }



}