package cn.wch.meshdemo

import android.graphics.Color
import android.view.Menu
import android.view.MenuItem
import android.view.View
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import cn.wch.meshdemo.adapter.NodeListAdapter
import cn.wch.meshdemo.base.AbstractActivity
import cn.wch.meshdemo.constant.IntentConstant
import cn.wch.meshdemo.core.NodeViewModelKt
import cn.wch.meshdemo.databinding.ActivityGroupDetailBinding
import cn.wch.meshdemo.storage.GroupStorageUtil
import cn.wch.meshdemo.storage.NodeItem
import cn.wch.meshdemo.storage.StorageUtil
import cn.wch.meshdemo.utils.ConvertUtil
import cn.wch.meshdemo.utils.LogTool
import cn.wch.meshlib.core.MeshServer
import com.yanzhenjie.recyclerview.OnItemMenuClickListener
import com.yanzhenjie.recyclerview.SwipeMenuBridge
import com.yanzhenjie.recyclerview.SwipeMenuCreator
import com.yanzhenjie.recyclerview.SwipeMenuItem
import java.util.*

class GroupDetailActivity: AbstractActivity() {

    lateinit var binding:ActivityGroupDetailBinding
    var groupAddress:Int?=null
    var groupName:String?=null
    var adapter:NodeListAdapter?=null

    var nodesInGroup:MutableList<NodeItem> = mutableListOf()

    override fun setView() {
        binding=ActivityGroupDetailBinding.inflate(layoutInflater)
        setContentView(binding.root)
        intent?.let {
            groupName=it.getStringExtra(IntentConstant.GROUP_NET_NAME)
            groupAddress=it.getIntExtra(IntentConstant.GROUP_NET_ADDRESS,-1);
        }
        binding.toolbar.title="${groupName}"
        binding.toolbar.subtitle= String.format(Locale.getDefault(),"Group地址:%04X",groupAddress)

        setSupportActionBar(binding.toolbar)
        supportActionBar?.let {
            it.setDisplayHomeAsUpEnabled(true)
            it.setHomeButtonEnabled(true)
        }
    }

    //https://blog.csdn.net/weixin_51362315/article/details/120009373
    //侧滑recyclerview
    override fun initWidget() {
        adapter = NodeListAdapter(object : NodeListAdapter.OnClickListener{
            override fun onClick(netAddress: Int?, name: String?) {

            }

            override fun onLongClick(netAddress: Int?, name: String?) {

            }
        })

        binding.container.rvNode.setSwipeMenuCreator(SwipeMenuCreator { leftMenu, rightMenu, position ->
            var menu:SwipeMenuItem=SwipeMenuItem(this)
                .setText("删除")
                .setBackgroundColor(Color.RED)
                .setWidth(200)//菜单宽度
                .setHeight(RecyclerView.LayoutParams.MATCH_PARENT);//菜单高度

            rightMenu.addMenuItem(menu)
        })
        binding.container.rvNode.setOnItemMenuClickListener(object :OnItemMenuClickListener{
            override fun onItemClick(menuBridge: SwipeMenuBridge?, adapterPosition: Int) {
                menuBridge?.closeMenu();
                //左菜单还是右菜单
                val direction = menuBridge!!.direction
                //菜单里item的下标
                val position = menuBridge!!.position
                LogTool.d("${direction} ${position} $adapterPosition")
                if(direction==-1 && position==0){
                    deleteNodeFromGroup(adapterPosition)
                }
            }
        })

        binding.container.rvNode.adapter=adapter
        binding.container.rvNode.layoutManager=
            LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
        binding.container.rvNode.addItemDecoration(DividerItemDecoration(this, DividerItemDecoration.VERTICAL))


        binding.container.btnSendData.setOnClickListener {
            sendData()
        }

        loadGroupDetail()

    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_group_detail,menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            android.R.id.home->{
                onBackPressed()
            }
            R.id.deleteGroup->{
                deleteGroup()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onDestroy() {
        super.onDestroy()
        LogTool.d("GroupDetailActivity destroy")
    }

    //重新加载Group下的各个节点
    public fun loadGroupDetail(){

        groupAddress?.let {
            nodesInGroup = GroupStorageUtil.getNodesInGroup(it) ?: return@let
            binding.container.tvLabel1.visibility= if(nodesInGroup.size==0) View.GONE else View.VISIBLE
            adapter?.loadAllNodes(nodesInGroup)
        }

    }

    private fun sendData(){
        if(groupAddress==null || groupAddress!!<0){
            LogTool.d("groupAddress address is null or negative")
            return
        }
        val dataString = binding.container.etNodeData.text.toString()
        if(dataString.isEmpty()){
            showToast("内容为空")
            return
        }
        var data:ByteArray?=null

        data = if(binding.container.swHex.isChecked){
            try {
                ConvertUtil.fromHexString2ByteArray(dataString)
            } catch (e: Exception) {
                showToast(e.message)
                return
            }
        }else{
            dataString.toByteArray()
        }

        NodeViewModelKt.model.rxjavaExecute(this,"正在发送数据",{
            MeshServer.getInstance().transferData(groupAddress!!,data)
        },{
            showToast("发送数据${if(it) "成功" else "失败"}")
        },{showToast(it.message)},{})
    }

    public fun deleteNodeFromGroup(position:Int){
        if(groupAddress==null){
            return
        }
        val nodeItem = nodesInGroup[position]
        NodeViewModelKt.model.rxjavaExecute(this,"从Group中删除节点",{
            MeshServer.getInstance().cancelSubscribe(nodeItem.nodeAddress,groupAddress!!)
        },{
            showToast("取消订阅${if(it) "成功" else "失败"}")
            if(it){
                GroupStorageUtil.removeNodeFromGroup(nodeItem.nodeAddress,groupAddress!!)
                loadGroupDetail()
            }
          },{showToast(it.message)},{})
    }

    public fun deleteGroup(){
        if(groupAddress==null){
            return
        }
        val nodesInGroup = GroupStorageUtil.getNodesInGroup(groupAddress!!) ?: return
        if(nodesInGroup.size!=0){
            showToast("请先删除Group下的节点")
            return
        }else{
            GroupStorageUtil.deleteGroup(groupAddress!!)
            showToast("删除成功")
            onBackPressed()
        }
    }
}