package cn.wch.meshdemo

import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import cn.wch.meshdemo.base.AbstractActivity
import cn.wch.meshdemo.constant.IntentConstant
import cn.wch.meshdemo.core.NodeViewModelKt
import cn.wch.meshdemo.databinding.ActivityNodeDetailBinding
import cn.wch.meshdemo.dialog.AddNodeToGroupDialog
import cn.wch.meshdemo.storage.GroupStorageUtil
import cn.wch.meshdemo.storage.StorageUtil
import cn.wch.meshdemo.utils.ConvertUtil
import cn.wch.meshdemo.utils.LogTool
import cn.wch.meshlib.core.MeshServer
import cn.wch.meshlib.frame.ack.AckType
import com.touchmcu.ui.DialogUtil
import java.util.*

class NodeDetailActivity : AbstractActivity() {

    lateinit var binding: ActivityNodeDetailBinding;
    var nodeAddress:Int?=null
    var nodeName:String?=null

    override fun setView() {
        binding=ActivityNodeDetailBinding.inflate(layoutInflater)
        setContentView(binding.root)

        intent?.let {
            nodeName=it.getStringExtra(IntentConstant.NODE_NET_NAME)
            nodeAddress=it.getIntExtra(IntentConstant.NODE_NET_ADDRESS,-1);
        }
        binding.toolbar.title="$nodeName"
        binding.toolbar.subtitle= String.format(Locale.getDefault(),"节点地址:%04X",nodeAddress)

        setSupportActionBar(binding.toolbar)
        supportActionBar?.let {
            it.setDisplayHomeAsUpEnabled(true)
            it.setHomeButtonEnabled(true)
        }
    }

    override fun initWidget() {


        binding.container.btnSendData.setOnClickListener {
            sendData()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_node_detail,menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.deleteNode->{
                startDeleteNode()
            }
            R.id.addToGroup->{
                addNodeToGroup()
            }
            android.R.id.home->{
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        super.onBackPressed()
    }

    override fun onDestroy() {
        super.onDestroy()
        LogTool.d("NodeDetailActivity destroy")
    }

    private fun startDeleteNode(){
        DialogUtil.getInstance().showSimpleDialog(this,"确定要删除此节点吗?","删除","取消",object :DialogUtil.IResult{
            override fun onContinue() {
                deleteNode()
            }

            override fun onCancel() {

            }
        })
    }

    private fun deleteNode(){
        if(nodeAddress==null || nodeAddress!!<0){
            LogTool.d("node address is null or negative")
            return
        }
        NodeViewModelKt.model.rxjavaExecute(this,"正在删除节点",{
            MeshServer.getInstance().deleteNode(nodeAddress!!)
        },{
          if(it.ackType==AckType.DELETE_NODE){
              showToast("删除节点${ConvertUtil.getNodeHexString(it.netAddress)}${if(it.statusCode==0) "成功" else "失败"}")
          }else{
              showToast("error ack")
          }
          if(it.statusCode==0){
              StorageUtil.deleteNode(nodeAddress!!)
              GroupStorageUtil.removeNodeFromAllGroups(nodeAddress!!)
              toNodeActivity()
          }
        },{showToast(it.message)},{

        })
    }

    private fun sendData(){
        if(nodeAddress==null || nodeAddress!!<0){
            LogTool.d("node address is null or negative")
            return
        }
        val dataString = binding.container.etNodeData.text.toString()
        if(dataString.isEmpty()){
            showToast("内容为空")
            return
        }
        var data:ByteArray?=null

        data = if(binding.container.swHex.isChecked){
            try {
                ConvertUtil.fromHexString2ByteArray(dataString)
            } catch (e: Exception) {
                showToast(e.message)
                return
            }
        }else{
            dataString.toByteArray()
        }

        NodeViewModelKt.model.rxjavaExecute(this,"正在发送数据",{
            MeshServer.getInstance().transferData(nodeAddress!!,data)
        },{
            showToast("发送数据${if(it) "成功" else "失败"}")
        },{showToast(it.message)},{})
    }

    private fun toNodeActivity(){
        startActivity(Intent(this,NodeActivity::class.java))
    }

    fun addNodeToGroup(){
        val dialog = AddNodeToGroupDialog.newInstance()
        dialog.isCancelable=false
        dialog.setSelectGroupListener(object :AddNodeToGroupDialog.OnSelectGroupListener{
            override fun onSelectGroup(address: Int) {
                LogTool.d("select group:0x${address.toString(16)}")
                nodeAddress?.let {
                    if(GroupStorageUtil.checkNodeHasExistInGroup(nodeAddress!!,address)){
                        showToast("本节点已经存在于该Group中!")
                    }else{
                        realAddToGroup(nodeAddress!!,address)
                    }
                }
            }
        })
        dialog.show(supportFragmentManager,AddNodeToGroupDialog::class.java.name)
    }

    fun realAddToGroup(nodeAddress:Int,groupAddress:Int){
        NodeViewModelKt.model.rxjavaExecute(this,"节点订阅Group",{
            MeshServer.getInstance().subscribe(nodeAddress, groupAddress)
        },{
            showToast("订阅${if(it) "成功" else "失败"}")
            if(it){
                GroupStorageUtil.addNodeToGroup(nodeAddress,groupAddress)
            }
        },{showToast(it.message)},{})
    }
}