package cn.wch.meshdemo

import android.annotation.SuppressLint
import android.app.Dialog
import android.bluetooth.BluetoothDevice
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanResult
import android.content.Intent
import android.os.Build
import android.view.*
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import cn.wch.meshdemo.base.ScanBaseActivity
import cn.wch.meshdemo.constant.IntentConstant
import cn.wch.meshdemo.databinding.ActivityScanBinding
import cn.wch.meshdemo.dialog.DeviceListAdapter
import cn.wch.meshdemo.utils.LogTool
import cn.wch.meshlib.util.BLEPermissionUtil
import java.util.ArrayList

class ScanActivity: ScanBaseActivity() {

    lateinit var binding: ActivityScanBinding;
    var adapter:DeviceListAdapter?=null
    var isScanning:Boolean=false;

    var filterDevices:ArrayList<String>?=null

    var sourceActivityClass:String?=null
    var targetActivityClass:String?=null
    var onlyScanConfiguredBLE:Boolean=false

    override fun setView() {
        binding=ActivityScanBinding.inflate(layoutInflater)
        setContentView(binding.root)
        binding.toolbar.title="搜索附近设备"
        setSupportActionBar(binding.toolbar)
        supportActionBar?.let {
            it.setDisplayHomeAsUpEnabled(true)
            it.setHomeButtonEnabled(true)
        }
    }

    override fun initWidget() {

        intent?.let {
            filterDevices=it.getStringArrayListExtra(IntentConstant.SCAN_FILTER_DEVICE)
            sourceActivityClass=it.getStringExtra(IntentConstant.SCAN_SOURCE_ACTIVITY);
            targetActivityClass=it.getStringExtra(IntentConstant.SCAN_TARGET_ACTIVITY);
            onlyScanConfiguredBLE=it.getBooleanExtra(IntentConstant.SCAN_ONLY_CONFIGURED,false)
        }

        adapter = DeviceListAdapter(object : DeviceListAdapter.OnClickListener {
            override fun onClick(device: BluetoothDevice) {
                setResult(device.address)
            }
        })
        binding.container.rvDevice.adapter=adapter
        binding.container.rvDevice.layoutManager=
            LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
        binding.container.rvDevice.addItemDecoration(DividerItemDecoration(this, DividerItemDecoration.VERTICAL))

        startScan()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_scan,menu)
        menu!!.findItem(R.id.startScan).isVisible=!isScanning
        menu!!.findItem(R.id.stopScan).isVisible=isScanning
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val itemId = item.itemId
        when(itemId){
            R.id.startScan->{
                startScan()
            }
            R.id.stopScan->{
                stopScan()
            }
            android.R.id.home->{
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        stopScan()
        super.onBackPressed()
    }

    override fun onDestroy() {
        super.onDestroy()
        LogTool.d("ScanActivity destroy")
    }

    @SuppressLint("MissingPermission")
    fun startScan(){
        //first check permission
        if(!checkPermissionsAllowed()){
            //request permissions
            requestPermission()
            return
        }
        //check bluetooth state
        if(!isBluetoothOpened()){
            //open bluetooth
            openBluetooth()
            return
        }
        //check location
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            if(!BLEPermissionUtil.isLocationEnable(this)){
                showToast("请打开位置信息开关")
                BLEPermissionUtil.requestLocationService(this)
                return
            }
        }
        //start scan
        try {
            clearAdapter()
            if(onlyScanConfiguredBLE){
                startScanConfiguredBLE(MyScanCallback())
            }else{
                startScanUnConfiguredBLE(MyScanCallback())
            }
            isScanning=true
            setProgressbarVisible(true)
            invalidateOptionsMenu()
        }catch (e:Exception){
            showToast(e.message)
        }

    }

    fun stopScan(){
        try {
            stopScanBLE()
            isScanning=false
            setProgressbarVisible(false)
            invalidateOptionsMenu()
        } catch (e: Exception) {
            LogTool.d(e.message)
        }
    }

    private fun setResult(mac: String){
        stopScan()
        LogTool.d("select device:$mac")
        //check source and target
        if(sourceActivityClass!=null && targetActivityClass!=null && sourceActivityClass.equals(targetActivityClass,false)){
            //same activity
            val intent = Intent()
            intent.putExtra(IntentConstant.SCAN_SELECT_DEVICE,mac)
            setResult(RESULT_OK,intent)
            finish()
        }else{
            if(targetActivityClass!=null){
                val intent = Intent(this, Class.forName(targetActivityClass))
                intent.putExtra(IntentConstant.SCAN_SELECT_DEVICE, mac)
                startActivity(intent)
            }
        }

    }

    private fun clearAdapter(){
        runOnUiThread{
            adapter?.clear()
        }
    }

    private fun setProgressbarVisible(visible:Boolean){
        runOnUiThread{
            binding.container.pbIndicator.visibility=if(visible) View.VISIBLE else View.INVISIBLE
        }
    }


    inner class MyScanCallback : ScanCallback(){
        @SuppressLint("MissingPermission")
        override fun onScanResult(callbackType: Int, result: ScanResult?) {
            val i = result?.device?.type
            if(i==BluetoothDevice.DEVICE_TYPE_LE){
                if(filterDevices!=null && filterDevices!!.size>0){
                    if(filterDevices!!.contains(result.device?.address)){
                        adapter?.update(result.device, result.rssi)
                    }
                }else{
                    adapter?.update(result.device, result.rssi)
                }

            }
        }
    }
}