package cn.wch.meshdemo.adapter

import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import cn.wch.meshdemo.databinding.ItemNodelistBinding
import cn.wch.meshdemo.storage.NodeItem
import cn.wch.meshdemo.utils.ConvertUtil
import java.util.*


class NodeListAdapter(var listener:OnClickListener): RecyclerView.Adapter<NodeListAdapter.MyViewHolder>() {

    private var deviceList: MutableList<NodeItem>? = null

    class MyViewHolder(binding: ItemNodelistBinding) : RecyclerView.ViewHolder(binding.root) {
        var netAddress: TextView =binding.netAddress
        var name: TextView =binding.name
        var state: TextView =binding.state
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyViewHolder {
        val inflate = ItemNodelistBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return MyViewHolder(inflate)
    }

    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        deviceList?.let {
            val nodeItem = it[position]
            holder.name.text=nodeItem.name
            holder.netAddress.text=String.format(Locale.getDefault(),"节点地址:%04X",nodeItem.nodeAddress)
            holder.state.text=when(nodeItem.state){
                null->"未知"
                true->"在线"
                false->"离线"
            }
            holder.itemView.setOnLongClickListener{
                listener.onLongClick(nodeItem.nodeAddress,nodeItem.name)
                true
            }
            holder.itemView.setOnClickListener {
                listener.onClick(nodeItem.nodeAddress,nodeItem.name)
            }
        }
    }

    override fun getItemCount(): Int {
        return deviceList?.size ?: 0
    }

    public fun loadAllNodes(items:MutableList<NodeItem>?){
        deviceList=items
        notifyDataSetChanged()
    }

    public fun updateNodeState(nodeAddress:Int,nodeState:Boolean?){
        deviceList?.let {
            for(i in deviceList!!.indices){
                val nodeItem = deviceList!![i]
                if(nodeItem.nodeAddress==nodeAddress){
                    deviceList!![i].state=nodeState
                    notifyItemChanged(i)
                    return@let
                }
            }
        }
    }

    public fun deleteNode(nodeAddress:Int){
        deviceList?.let {
            for(i in deviceList!!.indices){
                val nodeItem = deviceList!![i]
                if(nodeItem.nodeAddress==nodeAddress){
                    deviceList!!.removeAt(i)
                    notifyItemRemoved(i)
                    return@let
                }
            }
        }
    }

    interface OnClickListener{
        fun onClick(netAddress:Int?,name:String?)
        fun onLongClick(netAddress:Int?,name:String?)

    }


}