package cn.wch.meshdemo.base

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.view.WindowManager
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import cn.wch.meshdemo.utils.ToastUtil
import com.tbruyelle.rxpermissions2.RxPermissions

abstract class AbstractActivity : AppCompatActivity(){
    lateinit var rxpermission: RxPermissions

    val REQUEST_BLUETOOTH_CODE=121;

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        setView()
        init()
        initWidget()
    }

    abstract fun setView()

    abstract fun initWidget();

    private fun init(){
        rxpermission= RxPermissions(this)
    }

    private fun getRuntimePermissions():Array<String>{
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S){
            return arrayOf(Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN)
        }else{
            return arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION)
        }
    }

    fun checkPermissionsAllowed():Boolean{
        for (runtimePermission in getRuntimePermissions()) {
            if(ActivityCompat.checkSelfPermission(this,runtimePermission)!= PackageManager.PERMISSION_GRANTED){
                return false
            }
        }
        return true
    }

    fun requestPermission(operation:(()->Unit)?=null){
        rxpermission.requestEachCombined(*getRuntimePermissions()).subscribe {
            if (it.granted) {
                //allow permission
                if (operation != null) {
                    operation()
                }
            } else if (it.shouldShowRequestPermissionRationale) {
                showToast("请允许权限,否则APP不能正常运行")
            } else {
                //只要有一个选择：禁止，但选择“以后不再询问”，以后申请权限，不会继续弹出提示
                showToast("请到设置中打开权限")
            }
        }
    }

    fun isSupportBle(context: Context): Boolean {
        val packageManager = context.packageManager
        return BluetoothAdapter.getDefaultAdapter() != null && packageManager != null && packageManager.hasSystemFeature(
            PackageManager.FEATURE_BLUETOOTH_LE
        )
    }

    fun isBluetoothOpened():Boolean{
        return BluetoothAdapter.getDefaultAdapter()!=null && BluetoothAdapter.getDefaultAdapter().isEnabled
    }

    fun openBluetooth(){
        val intent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
        if(Build.VERSION.SDK_INT>= Build.VERSION_CODES.S){
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED) {
                showToast("please allow permission")
                ActivityCompat.requestPermissions(this,getRuntimePermissions(),123)
                return
            }
        }
        startActivityForResult(intent, REQUEST_BLUETOOTH_CODE)
    }

    fun showToast(message: String?) {
        runOnUiThread { ToastUtil.create(this,message) }
    }
}