package cn.wch.meshdemo.base

import android.bluetooth.BluetoothDevice
import cn.wch.meshdemo.utils.LogTool
import cn.wch.meshlib.core.ConCallback
import cn.wch.meshlib.core.MeshServer
import com.touchmcu.ui.DialogUtil

abstract class NodeBaseActivity :AbstractActivity(){

    protected var isConnected:Boolean=false


    protected fun connectBLE(mac:String){
        MeshServer.getInstance().connect(mac,object : ConCallback {
            override fun OnError(mac: String?, t: Throwable?) {
                showToast("error:${t?.message}")
                isConnected = false
                runOnUiThread {
                    DialogUtil.getInstance().hideLoadingDialog()
                    invalidateOptionsMenu()
                }
            }

            override fun OnConnecting(mac: String?) {
                isConnected = false
                runOnUiThread {
                    DialogUtil.getInstance().showLoadingDialog(this@NodeBaseActivity, "正在连接...")
                }
            }

            override fun OnConnectSuccess(mac: String?) {
                showToast("connect success")
                isConnected = true
                runOnUiThread {
                    DialogUtil.getInstance().hideLoadingDialog()
                    invalidateOptionsMenu()
                }
                onConnectBLESuccess(mac)
            }

            override fun OnInvalidDevice(mac: String?) {
                showToast("not target device")
                runOnUiThread{
                    disconnectBLE(false)
                }
            }

            override fun OnConnectTimeout(mac: String?) {
                showToast("connect timeout")
                isConnected = false
                runOnUiThread {
                    DialogUtil.getInstance().hideLoadingDialog()
                    invalidateOptionsMenu()
                }
            }

            override fun OnDisconnect(
                mac: String?,
                bluetoothDevice: BluetoothDevice?,
                status: Int
            ) {
                isConnected = false
                runOnUiThread {
                    DialogUtil.getInstance().hideLoadingDialog()
                    invalidateOptionsMenu()
                }
                showToast("disconnect")
                onBLEDisconnect(mac, status)
            }
        })
    }

    protected fun disconnectBLE(force:Boolean){
        try {
            MeshServer.getInstance().disconnect(force)
        } catch (e: Exception) {
            LogTool.d(e.message)
        }
    }

    protected fun isBLEConnected():Boolean{
        return MeshServer.getInstance().isConnected
    }

    protected open fun onConnectBLESuccess(mac: String?){

    }

    protected open fun onBLEDisconnect(mac: String?,status: Int){

    }
}