package cn.wch.meshdemo.core

import android.app.Activity
import cn.wch.meshdemo.utils.LogTool
import com.touchmcu.ui.DialogUtil
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.lang.Exception
import java.lang.ref.WeakReference

/**
 * 原则上不应该持有ui的引用，但此处为了方便
 */
class NodeViewModelKt {

    var activity:WeakReference<Activity>?=null;

    companion object{
        val model:NodeViewModelKt by lazy(mode = LazyThreadSafetyMode.SYNCHRONIZED){
            NodeViewModelKt()
        }
    }

    //attach weakReference
    fun attachToActivity(activity: WeakReference<Activity>){
        this.activity=activity;
    }

    //destroy weakReference
    fun detachActivity(){
        this.activity=null;
    }

    fun <T> rxjavaExecute(activity: Activity, message:String="", runnable:()->T?, next:(T)->Unit, error:(Throwable)->Unit, complete:()->Unit){
        Observable.create<T> {
            try {
                val t:T?=runnable()
                if(t==null){
                    it.onError(Throwable("fail"))
                    return@create
                }
                it.onNext(t)

            } catch (e: Exception) {
                LogTool.d("rxjava exception:${e.message}")
                it.onError(Throwable(e.message))
                return@create
            }
            it.onComplete()
        }.subscribeOn(Schedulers.newThread())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : Observer<T> {
                override fun onSubscribe(d: Disposable) {
                    activity.let {
                        DialogUtil.getInstance().showLoadingDialog(it,message)
                    }
                }
                override fun onNext(s: T) {
                    next(s)}
                override fun onError(e: Throwable) {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    error(e)
                }
                override fun onComplete() {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    complete()
                }
            })
    }


    fun <T> rxjavaExecuteAll(activity: Activity, message:String="", runnable:List<()->T?>, next:(T)->Unit, error:(Throwable)->Unit, complete:()->Unit){
        Observable.create<T> {

            for (function in runnable){
                try {
                    val t:T?=function()
                    if(t!=null){
                        it.onNext(t)
                    }
                }catch (e:Exception){
                    LogTool.d("rxjava exception:${e.message}")
                }
            }
//            try {
//                for (function in runnable) {
//                    val t:T?=function()
//                    if(t==null){
//                        it.onError(Throwable("fail"))
//                        return@create
//                    }
//                    it.onNext(t)
//                }
//
//            } catch (e: Exception) {
//                LogTool.d("rxjava exception:${e.message}")
//                it.onError(Throwable(e.message))
//                return@create
//            }
            it.onComplete()
        }.subscribeOn(Schedulers.newThread())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : Observer<T> {
                override fun onSubscribe(d: Disposable) {
                    activity.let {
                        DialogUtil.getInstance().showLoadingDialog(it,message)
                    }
                }
                override fun onNext(s: T) {
                    next(s)}
                override fun onError(e: Throwable) {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    error(e)
                }
                override fun onComplete() {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    complete()
                }
            })
    }

    fun <T> rxjavaExecuteAny(activity: Activity, message:String="", runnable:List<()->T?>, shouldContinue:(T)->Boolean, next:(T)->Unit, error:(Throwable)->Unit, complete:()->Unit){
        Observable.create<T> {
            try {
                for(i in runnable.indices){
                    val t:T?=runnable[i]()
                    if(t==null){
                        it.onError(Throwable("fail"))
                        return@create
                    }
                    if(!shouldContinue(t)){
                        it.onError(Throwable("fail"))
                        return@create
                    }
                    it.onNext(t)
                }

            } catch (e: Exception) {
                LogTool.d("rxjava exception:${e.message}")
                it.onError(Throwable(e.message))
                return@create
            }
            it.onComplete()
        }.subscribeOn(Schedulers.newThread())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : Observer<T> {
                override fun onSubscribe(d: Disposable) {
                    activity.let {
                        DialogUtil.getInstance().showLoadingDialog(it,message)
                    }
                }
                override fun onNext(s: T) {
                    next(s)}
                override fun onError(e: Throwable) {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    error(e)
                }
                override fun onComplete() {
                    activity.let {
                        DialogUtil.getInstance().hideLoadingDialog()
                    }
                    complete()
                }
            })
    }
}