package cn.wch.meshdemo.dialog

import android.annotation.SuppressLint
import android.bluetooth.BluetoothDevice
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import cn.wch.meshdemo.databinding.DialogDevicelistItemBinding
import java.util.*

class DeviceListAdapter(var listener:OnClickListener): RecyclerView.Adapter<DeviceListAdapter.MyViewHolder>() {

    private var deviceList: MutableList<BluetoothDevice>? = null
    private var rssiMap: MutableMap<String, Int>? = null

    init {
        deviceList= mutableListOf()
        rssiMap= mutableMapOf()
    }

    class MyViewHolder(binding: DialogDevicelistItemBinding) : RecyclerView.ViewHolder(binding.root) {
        var mac:TextView=binding.mac
        var name:TextView=binding.name
        var indicator:ImageView=binding.indicator
        var rssi:TextView=binding.rssi
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyViewHolder {
        val binding =
            DialogDevicelistItemBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return MyViewHolder(binding)
    }

    @SuppressLint("MissingPermission")
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        val device = deviceList!![position]
        holder.name.text = if (device.name == null) "null" else device.name
        holder.mac.text = String.format(Locale.getDefault(),"%s",device.address)

        val rssi = rssiMap!![device.address]!!
        holder.rssi.text = String.format(Locale.getDefault(),"%d dBm",rssi)

        //holder.indicator.getDrawable().setLevel(Math.abs(rssi))

        holder.itemView.setOnClickListener {
            listener.onClick(device)
        }
    }

    override fun onBindViewHolder(holder: MyViewHolder, position: Int, payloads: MutableList<Any>) {
        if (payloads.isEmpty()) {
            onBindViewHolder(holder, position)
        } else {
            if (payloads[0] != null) {
                if (payloads[0] is RSSIMSG) {
                    val msg = payloads[0] as RSSIMSG
                    holder.rssi.text = String.format(Locale.getDefault(),"%d dBm",(msg.rssi))
                    //holder.indicator.getDrawable().setLevel(Math.abs(msg.rssi))
                }
            }
        }
    }

    override fun getItemCount(): Int {
        return deviceList?.size ?: 0
    }

    fun clear(){
        deviceList?.clear()
        notifyDataSetChanged()
    }

    fun update(device: BluetoothDevice, rssi: Int) {
        if (deviceList == null) {
            return
        }
        for (i in deviceList!!.indices) {
            val bluetoothDevice: BluetoothDevice = deviceList!!.get(i)
            if (bluetoothDevice.address.equals(device.address, ignoreCase = true)) {
                //更新rssi值
                rssiMap!!.put(device.address, rssi)
                notifyItemChanged(i, RSSIMSG(bluetoothDevice.address, rssi))
                return
            }
        }
        deviceList!!.add(device)
        rssiMap!!.put(device.address, rssi)
        notifyItemInserted(itemCount)
    }

    class RSSIMSG(var address: String, var rssi: Int)

    interface OnClickListener{
        fun onClick(device: BluetoothDevice)
    }
}