package cn.wch.meshdemo.dialog

import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import cn.wch.meshdemo.databinding.DialogCreateGroupBinding
import cn.wch.meshdemo.storage.GroupStorageUtil
import cn.wch.meshdemo.storage.StorageUtil
import cn.wch.meshdemo.utils.ConvertUtil
import cn.wch.meshdemo.utils.ToastUtil

class InputGroupInfoDialog:BaseDialogFragment() {

    var binding:DialogCreateGroupBinding?=null
    var listener:OnClickListener?=null
    companion object{
        fun newInstance(): InputGroupInfoDialog {
            val args = Bundle()
            val fragment = InputGroupInfoDialog()
            fragment.arguments = args
            return fragment
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = DialogCreateGroupBinding.inflate(inflater)
        initWidget()
        return binding!!.root
    }


    override fun onDestroyView() {
        super.onDestroyView()
        binding=null
    }

    private fun initWidget(){
        binding!!.cancel.setOnClickListener {
            dismiss()
        }
        binding!!.confirm.setOnClickListener {
            check()
        }

        binding!!.etAddress.addTextChangedListener(object:TextWatcher{
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {

            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {

            }

            override fun afterTextChanged(s: Editable?) {
                val address = s.toString()
                if(address.isEmpty()){
                    binding!!.tiLayout.error="cannot be empty!"
                    return
                }
                if(!address.matches(Regex("([0-9|a-f|A-F]{2}){2}"))){
                    binding!!.tiLayout.error="invalid address"
                    return
                }
                binding!!.tiLayout.error=null
            }
        } )
    }

    private fun check(){
        val name = binding!!.etName.text.toString()
        val address = binding!!.etAddress.text.toString()

        //check address
        if(!address.matches(Regex("([0-9|a-f|A-F]{2}){2}"))){
            showToast("地址必须为合法的2字节")
            return
        }
        //check name
        if(name.isNullOrEmpty()){
            showToast("Group名称为空")
            return
        }

        val addressData = address.toInt(16)
        if((addressData)<0x0000C000){
            showToast("Group地址应大于C000")
            return
        }

        if(GroupStorageUtil.checkGroupHasExist(addressData)){
            //已经存在
            showToast("Group地址已经存在")
            return
        }
        listener?.onCreateGroup(addressData,name)
        dismiss()
    }

    public fun setCreateGroupListener(listener:OnClickListener){
        this.listener=listener
    }

    public interface OnClickListener{
        fun onCreateGroup(address:Int,name:String)
    }

    fun showToast(message:String){
        ToastUtil.create(requireActivity(),message)
    }
}