package cn.wch.meshdemo.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import cn.wch.meshdemo.adapter.GroupListAdapter
import cn.wch.meshdemo.core.NodeViewModel
import cn.wch.meshdemo.databinding.FragmentGroupsBinding
import cn.wch.meshdemo.dialog.InputGroupInfoDialog
import cn.wch.meshdemo.utils.LogTool

class GroupsFragment:Fragment() {

    var binding:FragmentGroupsBinding?=null
    var adapter:GroupListAdapter?=null

    var viewModel:NodeViewModel?=null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentGroupsBinding.inflate(inflater)
        initWidget()
        return binding!!.root
    }


    override fun onDestroyView() {
        super.onDestroyView()
        binding=null
        LogTool.d("GroupsFragment onDestroyView")
    }

    private fun initWidget() {
        viewModel = ViewModelProvider(requireActivity()).get(NodeViewModel::class.java)

        adapter= GroupListAdapter(object :GroupListAdapter.OnClickListener{
            override fun onClick(groupAddress: Int?, name: String?) {

                viewModel!!.toGroupDetail(groupAddress,name)
            }

            override fun onLongClick(groupAddress: Int?, name: String?) {

            }
        })
        binding!!.rvGroup.adapter=adapter
        binding!!.rvGroup.layoutManager=
            LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        binding!!.rvGroup.addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.VERTICAL))


        binding!!.fab.setOnClickListener {
            showCreateGroupDialog()
        }

        viewModel!!.groupList.observe(viewLifecycleOwner){
            adapter?.loadAllGroups(it)
        }

    }

    override fun onStart() {
        super.onStart()
        viewModel!!.refreshAllGroupState()
    }

    override fun onResume() {
        super.onResume()
    }

    private fun showCreateGroupDialog(){
        val dialog = InputGroupInfoDialog.newInstance()
        dialog.isCancelable=false
        dialog.setCreateGroupListener(object :InputGroupInfoDialog.OnClickListener{
            override fun onCreateGroup(address: Int, name: String) {
                LogTool.d("create group:0x${address.toString(16)} $name")
                viewModel!!.addGroup(name,address)
            }
        })
        dialog.show(parentFragmentManager,InputGroupInfoDialog::class.java.name)
    }

}