package cn.wch.meshdemo.storage

import cn.wch.meshdemo.utils.LogTool
import com.tencent.mmkv.MMKV
import java.util.*

object GroupStorageUtil {

    /*记录Group相关参数*/

    private val MMKV_GROUP="mmkv_group";

    fun putGroup(groupItem: GroupItem){
        val defaultMMKV = MMKV.mmkvWithID(MMKV_GROUP)
        defaultMMKV.encode(createGroupAddress(groupItem.address),groupItem)
    }

    fun getGroup(groupAddress:Int):GroupItem?{
        val defaultMMKV = MMKV.mmkvWithID(MMKV_GROUP)
        return defaultMMKV.decodeParcelable(createGroupAddress(groupAddress), GroupItem::class.java)
    }


    fun deleteGroup(groupAddress:Int){
        val defaultMMKV = MMKV.mmkvWithID(MMKV_GROUP)
        defaultMMKV.remove(createGroupAddress(groupAddress))
    }

    fun addNodeToGroup(nodeAddress:Int,groupAddress:Int){
        val group = getGroup(groupAddress)
        if(group!=null){

            val nodeAddressItem = group.nodeAddressItem
            val newItem = createGroupAddress(nodeAddress)
            if(!nodeAddressItem.contains(newItem)){
                nodeAddressItem.add(newItem)
            }
            updateGroupItems(groupAddress,nodeAddressItem)
        }
    }

    fun removeNodeFromGroup(nodeAddress:Int,groupAddress:Int){
        val group = getGroup(groupAddress)
        if(group!=null){
            val nodeAddressItem = group.nodeAddressItem
            val newItem = createGroupAddress(nodeAddress)
            if(nodeAddressItem.contains(newItem)){
                nodeAddressItem.remove(newItem)
                updateGroupItems(groupAddress,nodeAddressItem)
            }
        }
    }

    fun removeNodeFromAllGroups(nodeAddress:Int){
        LogTool.d("remove node 0x${nodeAddress.toString(16)} from all group")
        val defaultMMKV = MMKV.mmkvWithID(MMKV_GROUP)
        val allKeys = defaultMMKV.allKeys()
        allKeys?.forEach {
            val gAddress = parseGroupAddress(it)
            removeNodeFromGroup(nodeAddress,gAddress)
        }
    }


    private fun updateGroupItems(groupAddress:Int, nodeAddressItems: MutableList<String>){
        val group = getGroup(groupAddress)
        if(group!=null){
            putGroup(GroupItem(group.name, group.address, nodeAddressItems))
        }
    }


    fun checkGroupHasExist(groupAddress:Int):Boolean{
        val allNode = getAllGroup()
        for (groupItem in allNode) {
            if(groupItem.address==groupAddress){
                return true
            }
        }
        return false
    }


    fun checkNodeHasExistInGroup(nodeAddress:Int,groupAddress:Int):Boolean{
        val group = getGroup(groupAddress)
        if(group!=null){
            return group.nodeAddressItem.contains(createGroupAddress(nodeAddress))
        }
        return false
    }



    fun getAllGroup():List<GroupItem>{
        val defaultMMKV = MMKV.mmkvWithID(MMKV_GROUP)
        val allKeys = defaultMMKV.allKeys()
        val mutableList = mutableListOf<GroupItem>()
        allKeys?.forEach {
            val item = getGroup(parseGroupAddress(it))
            if(item!=null){
                mutableList.add(item)
            }
        }
        return mutableList
    }

    public fun getNodesInGroup(groupAddress:Int):MutableList<NodeItem>?{
        val group = getGroup(groupAddress) ?: return null
        var list:MutableList<NodeItem> = mutableListOf()
        group.nodeAddressItem.forEach {
            val node = StorageUtil.getNode(it)
            if(node!=null){
                list.add(node)
            }
        }
        return list;
    }

    private fun createGroupAddress(nodeAddress: Int):String{
        return String.format(Locale.getDefault(),"%04X",nodeAddress)
    }

    private fun parseGroupAddress(nodeAddress:String):Int{
        return nodeAddress.toInt(16)
    }
}