package cn.wch.meshdemo.storage

import cn.wch.meshdemo.utils.LogTool
import com.tencent.mmkv.MMKV
import java.util.*

object StorageUtil {

    private val IVINDEX="iv_index";
    private val FLAG="flag";
    private val NODE_CONFIG_KEY="config_key";

    private val MMKV_NODE="mmkv_node";

    private val MMKV_OTHER="mmkv_other";



    fun putNode(nodeItem: NodeItem){
        val defaultMMKV = MMKV.mmkvWithID(MMKV_NODE)
        defaultMMKV.encode(createNodeAddress(nodeItem.nodeAddress),nodeItem)
    }

    private fun getNode(nodeAddress:Int):NodeItem?{
        val defaultMMKV = MMKV.mmkvWithID(MMKV_NODE)
        return defaultMMKV.decodeParcelable(createNodeAddress(nodeAddress), NodeItem::class.java)
    }

    fun getNode(nodeAddress:String):NodeItem?{
        val defaultMMKV = MMKV.mmkvWithID(MMKV_NODE)
        return defaultMMKV.decodeParcelable(nodeAddress, NodeItem::class.java)
    }

    fun deleteNode(nodeAddress:Int){
        LogTool.d("delete node 0x${nodeAddress.toString(16)}")
        val defaultMMKV = MMKV.mmkvWithID(MMKV_NODE)
        defaultMMKV.remove(createNodeAddress(nodeAddress))
    }

    fun updateNodeState(nodeAddress:Int, state:Boolean){
        val node = getNode(nodeAddress)
        if(node!=null){
            putNode(NodeItem(node.name,node.nodeAddress,state))
        }
    }

    fun updateNodeName(nodeAddress:Int, name:String){
        val node = getNode(nodeAddress)
        if(node!=null){
            putNode(NodeItem(name,node.nodeAddress,node.state))
        }
    }

    fun checkNodeHasExist(nodeAddress:Int):Boolean{
        val allNode = getAllNode()
        for (nodeItem in allNode) {
            if(nodeItem.nodeAddress==nodeAddress){
                return true
            }
        }
        return false
    }

    fun getAllNode():List<NodeItem>{
        val defaultMMKV = MMKV.mmkvWithID(MMKV_NODE)
        val allKeys = defaultMMKV.allKeys()
        val mutableList = mutableListOf<NodeItem>()
        allKeys?.forEach {
            val item = getNode(parseNodeAddress(it))
            if(item!=null){
                mutableList.add(item)
            }

        }
        return mutableList
    }


    /*记录其他信息iv_index、flag、key*/

    public fun getIvIndex():Int{
        return MMKV.mmkvWithID(MMKV_OTHER).decodeInt(IVINDEX,0)
    }

    public fun setIvIndex(ivIndex:Int):Boolean{
        LogTool.d("setIvIndex:${ivIndex}")
        return MMKV.mmkvWithID(MMKV_OTHER).encode(IVINDEX,ivIndex)
    }

    public fun getFlag():Int{
        return MMKV.mmkvWithID(MMKV_OTHER).decodeInt(FLAG,0)
    }

    public fun setFlag(flag:Int):Boolean{
        LogTool.d("setFlag:${flag}")
        return MMKV.mmkvWithID(MMKV_OTHER).encode(FLAG,flag)
    }

    public fun getConfigKey():String?{
        return MMKV.mmkvWithID(MMKV_OTHER).decodeString(NODE_CONFIG_KEY)
    }

    public fun setConfigKey(key:String):Boolean{
        LogTool.d("setConfigKey:${key}")
        return MMKV.mmkvWithID(MMKV_OTHER).encode(NODE_CONFIG_KEY,key)
    }


    private fun createNodeAddress(nodeAddress: Int):String{
        return String.format(Locale.getDefault(),"%04X",nodeAddress)
    }

    private fun parseNodeAddress(nodeAddress:String):Int{
        return nodeAddress.toInt(16)
    }



}