package cn.wch.meshlib.frame;

import androidx.annotation.NonNull;

import java.nio.ByteBuffer;

import cn.wch.meshlib.constant.Command;

public class CommandGenerateUtil {

    private static final int PERIPHERAL_CMD_LEN=1;
    private static final int PROVISION_NET_KEY_LEN=16;
    private static final int ADDRESS_LEN=2;
    private static final int UPDATE_ADDRESS_LEN=2;

    private static final int STATUS_CODE_LEN=1;
    private static final int IV_INDEX_LEN=4;
    private static final int FLAG_LEN=1;
    private static final int CONTROL_CODE_LEN=1;

    public static byte[] getProvisionInfoCommand(byte control_code,byte[] iv_index,byte flag){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+CONTROL_CODE_LEN+IV_INDEX_LEN+FLAG_LEN);
        byteBuffer.put(Command.CMD_PROVISION_INFO);
        byteBuffer.put(control_code);
        byteBuffer.put(trimByteArray(iv_index,IV_INDEX_LEN));
        byteBuffer.put(flag);
        return byteBuffer.array();
    }

    public static byte[] getProvisionCommand(byte[] key,byte[] address){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+PROVISION_NET_KEY_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_PROVISION);
        byteBuffer.put(trimByteArray(key,PROVISION_NET_KEY_LEN));
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        return byteBuffer.array();
    }

    public static byte[] getDeleteNodeCommand(byte[] address){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_DELETE_NODE);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        return byteBuffer.array();
    }

//    public static byte[] getDeleteNodeStorageCommand(){
//        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN);
//        byteBuffer.put(Command.CMD_DELETE_NODE_INFO);
//        return byteBuffer.array();
//    }

    public static byte[] getQueryNodeStateCommand(byte[] address){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_ASK_STATUS);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        return byteBuffer.array();
    }

    public static byte[] getTransferDataCommand(byte[] address,byte[] data){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN+data.length);
        byteBuffer.put(Command.CMD_TRANSFER);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        byteBuffer.put(data);
        return byteBuffer.array();
    }

    public static byte[] getOTAQueryNodeImageCommand(byte[] address){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_IMAGE_INFO);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        return byteBuffer.array();
    }

    public static byte[] getOTAUpdateCommand(byte[] address,int flashAddress,byte[] data,int offset,int length){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN+UPDATE_ADDRESS_LEN+length);
        byteBuffer.put(Command.CMD_UPDATE);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        byteBuffer.put((byte) (flashAddress & 0x000000ff));
        byteBuffer.put((byte) ((flashAddress & 0x0000ff00) >>8));
        byteBuffer.put(data,offset,length);
        return byteBuffer.array();
    }

    public static byte[] getOTAVerifyCommand(byte[] address,int flashAddress,byte[] data,int offset,int length){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN+UPDATE_ADDRESS_LEN+length);
        byteBuffer.put(Command.CMD_VERIFY);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        byteBuffer.put((byte) (flashAddress & 0x000000ff));
        byteBuffer.put((byte) ((flashAddress & 0x0000ff00) >>8));
        byteBuffer.put(data,offset,length);
        return byteBuffer.array();
    }

    public static byte[] getOTAEndCommand(byte[] address){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_END_INFO);
        byteBuffer.put(trimByteArray(flipByteArray(address),ADDRESS_LEN));
        return byteBuffer.array();
    }

    public static byte[] getSubscribeNodeCommand(byte[] address1,byte control_code,byte[] address2){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN+ADDRESS_LEN+CONTROL_CODE_LEN+ADDRESS_LEN);
        byteBuffer.put(Command.CMD_SET_SUB);
        byteBuffer.put(trimByteArray(flipByteArray(address1),ADDRESS_LEN));
        byteBuffer.put(control_code);
        byteBuffer.put(trimByteArray(flipByteArray(address2),ADDRESS_LEN));
        return byteBuffer.array();
    }



    public static byte[] getResetCommand(){
        ByteBuffer byteBuffer=ByteBuffer.allocate(PERIPHERAL_CMD_LEN);
        byteBuffer.put(Command.CMD_LOCAL_RESET);
        return byteBuffer.array();
    }

    private static byte[] trimByteArray(byte[] data, int except){
        if(data==null){
            return new byte[except];
        }
        if(except==data.length){
            return data;
        }
        byte[] temp=new byte[except];
        System.arraycopy(data,0,temp,0,Math.min(data.length,except));
        return temp;
    }

    //
    public static byte[] flipByteArray(@NonNull byte[] data){
        byte[] target=new byte[data.length];

        for(int i=0;i<data.length;i++){
            target[i]=data[data.length-1-i];
        }
        return target;
    }
}
